package tech.xiongweixp.smarttank;

import android.app.Activity;
import android.util.Log;
import android.view.KeyEvent;
import android.os.Bundle;
import android.view.WindowManager;
import com.google.android.exoplayer2.DefaultLoadControl;
import com.google.android.exoplayer2.ExoPlayer;
import com.google.android.exoplayer2.MediaItem;
import com.google.android.exoplayer2.source.MediaSource;
import com.google.android.exoplayer2.source.rtsp.RtspMediaSource;
import com.google.android.exoplayer2.ui.StyledPlayerView;
import org.eclipse.paho.mqttv5.client.IMqttToken;
import org.eclipse.paho.mqttv5.client.MqttAsyncClient;
import org.eclipse.paho.mqttv5.client.MqttConnectionOptions;
import org.eclipse.paho.mqttv5.client.persist.MemoryPersistence;
import org.eclipse.paho.mqttv5.common.MqttException;
import org.eclipse.paho.mqttv5.common.MqttMessage;

/**
 * @author xiong
 */
public class MainActivity extends Activity {
    private static final String TAG = "TankController";
    private static final int KEY_UP = KeyEvent.KEYCODE_DPAD_UP;
    private static final int KEY_DONW = KeyEvent.KEYCODE_DPAD_DOWN;
    private static final int KEY_LEFT = KeyEvent.KEYCODE_DPAD_LEFT;
    private static final int KEY_RIGHT = KeyEvent.KEYCODE_DPAD_RIGHT;
    private static final int KEY_A = KeyEvent.KEYCODE_BUTTON_A;
    private static final int KEY_B = KeyEvent.KEYCODE_BUTTON_B;
    private static String SERVER = "192.168.31.98";
    private static final int MQTTPORT = 1883;
    private static final String ID = "ControllerApp";
    private static final String TOPIC = "carcontrol";
    private MqttAsyncClient mqttClient;
    private boolean moving = false;
    private StyledPlayerView playerView;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // 防止关闭屏幕
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        setContentView(R.layout.activity_main);
        connectToMqtt();
        createPlayer();
    }

    private void createPlayer(){
        //创建播放器
        //创建播放器
        playerView = findViewById(R.id.playerView);
        DefaultLoadControl dlc = new DefaultLoadControl.Builder()
                .setBufferDurationsMs(100,
                                      100,
                                100,
                        100)
                .createDefaultLoadControl();
        ExoPlayer player = new ExoPlayer.Builder(this).setLoadControl(dlc).build();
        playerView.setPlayer(player);
        // 隐藏播放器控制按钮
        playerView.setUseController(false);
        //创建MediaSource
        MediaSource mediaSource = new RtspMediaSource.Factory()
                .createMediaSource(
                        MediaItem.fromUri("rtsp://192.168.31.98:8554/mystream"));
        //准备播放器
        player.prepare(mediaSource);
        //开始播放
        player.setPlayWhenReady(true);
    }


    //写一个方法，建立mqtt连接
    private void connectToMqtt() {
        IMqttToken token;
        try {
            String broker = "tcp://"+SERVER+":"+MQTTPORT;
            MemoryPersistence persistence = new MemoryPersistence();
            MqttConnectionOptions connOpts = new MqttConnectionOptions();
            connOpts.setCleanStart(false);
            mqttClient =  new MqttAsyncClient(broker, ID, persistence);
            token = mqttClient.connect(connOpts);
            token.waitForCompletion();
        }catch (MqttException e)   {
            Log.e(TAG, e.getMessage());
        }
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        switch (event.getAction()){
            case KeyEvent.ACTION_DOWN:
                switch (event.getKeyCode()){
                    case KEY_UP:
                        if (moving) {
                            break;
                        }
                        Log.i(TAG, "按键 上 按下");
                        moving = true;
                        sendCmd("f");
                        break;
                    case KEY_DONW:
                        if (moving) {
                            break;
                        }
                        Log.i(TAG, "按键 下 按下");
                        moving = true;
                        sendCmd("b");
                        break;
                    case KEY_LEFT:
                        if (moving) {
                            break;
                        }
                        Log.i(TAG, "按键 左 按下");
                        moving = true;
                        sendCmd("l");
                        break;
                    case KEY_RIGHT:
                        if (moving) {
                            break;
                        }
                        Log.i(TAG, "按键 右 按下");
                        moving = true;
                        sendCmd("r");
                        break;
                    case KEY_A:
                        Log.i(TAG, "按键 A 按下");
                        break;
                    case KEY_B:
                        Log.i(TAG, "按键 B 按下");
                        break;
                    default:
                        break;
                }
                break;
            case KeyEvent.ACTION_UP:
                switch (event.getKeyCode()){
                    case KEY_UP:
                    case KEY_DONW:
                    case KEY_LEFT:
                    case KEY_RIGHT:
                        Log.i(TAG, "方向键松开");
                        moving = false;
                        sendCmd("p");
                        break;
                    case KEY_A:
                        Log.i(TAG, "A键松开");
                        sendCmd("u");
                        break;
                    case KEY_B:
                        Log.i(TAG, "B键松开");
                        sendCmd("d");
                        break;
                    default:
                        break;
                }
                break;
            default:
                break;
        }

        return true;
    }

    private void sendCmd(String cmd){
        MqttMessage message = new MqttMessage(cmd.getBytes());
        try {
            // 判断mqttClient是否已连接再发送message
            if (mqttClient!=null && mqttClient.isConnected()) {
                mqttClient.publish(TOPIC, message);
            }
        } catch (MqttException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        try {
            mqttClient.disconnect();
        } catch (MqttException e) {
            e.printStackTrace();
        }
        //释放播放器
        playerView.getPlayer().release();
    }

}